from rocolib.api.ports import Port
from rocolib.utils.utils import prefix as prefixString
from rocolib.utils.numsym import deg2rad, dot
from rocolib.utils.transforms import RotateX, RotateZ

class FacePort(Port):
  def __init__(self, parent, graph, face):
    Port.__init__(self, parent, {})
    self.graph = graph
    self.face = face

  def update(self, newparent, oldcomposable, newcomposable, prefix):
    self.setParent(newparent)
    if oldcomposable == self.graph:
      self.face = prefixString(prefix, self.face)
      self.graph = newcomposable

  def getFace(self):
    return self.graph.getFace(self.face)

  def getFaceName(self):
    return self.face

  def toString(self):
    return str(self.face)

  def canMate(self, otherPort):
    try:
      if (otherPort.getDecoration() is not None):
        return True
    except AttributeError:
      pass
    try:
      if (otherPort.getFaceName() is not None):
        return True
    except:
      pass

  def attachTo(self, toPort, **kwargs):
    # if connecting from face to face, merge them in order
    if isinstance(toPort, FacePort):
      face1 = self.graph.getFace(self.getFaceName())
      face2 = self.graph.getFace(toPort.getFaceName())

      transform = kwargs.get("transform", None)
      if transform is not False:
          transform = dot(RotateX(deg2rad(kwargs.pop("flip", 0) and 180)), RotateZ(deg2rad(kwargs.pop("rotate", 0))))

      self.graph.mergeFace(face1.name, face2.name, transform=transform,
                                                   offset=kwargs.pop("offset", (0,0)),
                                                   copyDecorations=kwargs.pop("copyDecorations", False))
