from rocolib.api.components import FoldedComponent
from rocolib.api.composables.graph.Face import Rectangle

class SimpleRectBeam(FoldedComponent):
  def define(self):
    self.addParameter("length", 100, paramType="length")
    self.addParameter("width", 20, paramType="length")
    self.addParameter("depth", 50, paramType="length")

    self.addParameter("addTabs", True, valueType="bool")

    for i in range(4):
      self.addEdgeInterface("topedge%d" % i, "r%d.e2" % i, ["width", "depth"][i % 2])
      self.addEdgeInterface("botedge%d" % i, "r%d.e0" % i, ["width", "depth"][i % 2])
      self.addFaceInterface("face%d" % i, "r%d" % i)
    self.addEdgeInterface("slotedge", "r3.e1", "length")
    self.addEdgeInterface("tabedge", "r0.e3", "length")

  def assemble(self):
    length = self.getParameter("length")
    width = self.getParameter("width")
    depth = self.getParameter("depth")

    rs = []
    rs.append(Rectangle("", width, length))
    rs.append(Rectangle("", depth, length))
    rs.append(Rectangle("", width, length))
    rs.append(Rectangle("", depth, length))

    fromEdge = None
    for i in range(4):
      self.attachEdge(fromEdge, rs[i], "e3", prefix="r%d"%i, angle=90)
      fromEdge = 'r%d.e1' % i
      self.setFaceInterface("face%d" % i, "r%d" % i)

    tabEdge, slotEdge = "r0.e3", "r3.e1"

    if self.getParameter("addTabs"):
        self.addTab(tabEdge, slotEdge, angle= 90, width=min(10, depth))

if __name__ == "__main__":
  SimpleRectBeam.test()
